#include <stdio.h>
#include <stdlib.h>

/* Realizzare la funzione f(a, n) in base alla specifica seguente.  La
   funzione accetta come parametro un array a[] di interi avente
   lunghezza n. 

   Si garantisce che l'array sara' sempre non vuoto (n > 0) e che gli
   elementi di a[] possano valere solo 0 oppure 1.

   La funzione deve restituire un NUOVO array r[] di lunghezza n. Il
   contenuto di r[] deve essere definito come segue:

   - per ogni i < 4, r[i] deve valere 0;

   - altrimenti, per ogni i>=4, r[i] deve essere il valore che compare
     il maggior numero di volte nell'insieme {a[i-4], a[i-3], a[i-2],
     a[i-1], a[i]}. In altre parole, r[i] deve essere il valore che
     compare il maggior numero di volte nel sottovettore a[i-4,...,i]
    
   Si ricordi che, poiche' l'array a[] puo' contenere solo valori 0
   oppure 1, anche il risultato r[] conterra' solo valori 0 oppure 1.

 */
int *f(int *a, int n)
{
    int i;
    int *r = (int*)malloc(n*sizeof(int));
    /* Azzerare gli elementi r[0], r[1], r[2] e r[3] in questo punto
       sarebbe stato errato: è infatti possibile che n sia minore di
       quattro (cioè che l'array a[] abbia meno di quattro elementi),
       per cui non è detto che r[0], ... r[3] esistano sempre. La
       soluzione corretta è di azzerarli all'interno del ciclo "for",
       in modo da tenere conto solo di quelli effettivamente
       presenti. */
    for (i=0; i<n; i++) {
        if (i<4) {
            r[i] = 0;
        } else {
            /* contiamo quanti 1 ci sono in a[i-4, ... i] */
            int c = 0;
            int k;
            for (k=-4; k<=0; k++) {
                c += a[i+k];
            }
            if (c<3) { /* ci sono piu' 0 che 1 */
                r[i] = 0;
            } else {
                r[i] = 1;
            }
        }
    }
    return r;
}

int array_equal(int *a, int *b, int n)
{
    if ((a == NULL) || (b == NULL)) {
        return (a == b);
    } else {
        int i;
        for (i=0; i<n; i++) {
            if (a[i] != b[i])
                return 0;
        }
        return 1;
    }
}         

void print_array(int *a, int n)
{
    if (a == NULL) {
        printf("NULL");
    } else {
        int i;
        printf("[");
        for (i=0; i<n; i++) {
            printf("%d", a[i]);
            if (i<n-1) {
                printf(", ");
            }
        }
        printf("]");
    }
}

void test(int *a, int *expect, int n)
{
    int *r = f(a, n);
    if (array_equal(r, expect, n)) {
        printf("Test OK\n");
    } else {
        printf("Test FALLITO\n");
    }
    free(r);
}

#define ARRAY_LEN(a) (sizeof(a)/sizeof(a[0]))

int main( void )
{
    int a1[] = {0, 1, 0};
    int r1[] = {0, 0, 0};

    int a2[] = {0, 1, 1, 1, 0, 0, 1, 0, 0, 1, 1, 1};
    int r2[] = {0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1};

    int a3[] = {0, 1, 0, 1, 0};
    int r3[] = {0, 0, 0, 0, 0};

    int a4[] = {0, 1, 0, 1, 1};
    int r4[] = {0, 0, 0, 0, 1};

    int a5[] = {0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0};
    int r5[] = {0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0};
    
    printf("=== Inizio test ===\n");
    test(a1, r1, ARRAY_LEN(a1));
    test(a2, r2, ARRAY_LEN(a2));
    test(a3, r3, ARRAY_LEN(a3));
    test(a4, r4, ARRAY_LEN(a4));
    test(a5, r5, ARRAY_LEN(a5));
    printf("=== Fine test ===\n");
    return 0;
}
