#include <stdio.h>
#include <stdlib.h>

struct list {
    int val;
    struct list *next;
};

/* Dichiarazione di alcune delle funzioni viste a lezione; le funzioni
   sono definite sotto (non devono essere implementate), e possono
   essere usate se serve. */
struct list *list_create(int v, struct list *t);
int list_length(struct list *L);
void list_destroy(struct list *L);
int is_empty(struct list *L);
int list_equal(struct list *L1, struct list *L2);
void list_print(struct list *L);

/* FUNZIONE DA REALIZZARE
   ======================

   Realizzare la funzione f(L) in base alla specifica seguente. La
   funzione accetta come parametro una lista L di interi. La lista e'
   garantita essere non vuota.

   1. La funzione deve determinare il valore minimo e massimo presente
   in L.

   2. Successivamente, deve modificare L aggiungendo due nodi
   all'inizio: il primo contenente il valore minimo, e il secondo
   contenente il valore massimo precedentemente calcolati.
   
   3. La funzione restituira' un puntatore all'inizio della lista
   cosi' modificata.

   Ad esempio, se L = (13, 2, -7, 21, -7) la funzione deve modificare
   L in modo che alla fine contenga i nodi (-7, 21, 13, 2, -7, 21,
   -7), e restituire un puntatore all'inizio della lista cosi'
   modificata.

*/
struct list *f(struct list *L)
{
    int min = L->val;
    int max = L->val;
    struct list *nodo_min, *nodo_max, *tmp;

    for (tmp = L; tmp != NULL; tmp = tmp->next) {
        if (min > tmp->val)
            min = tmp->val;

        if (max < tmp->val)
            max = tmp->val;
    }

    nodo_max = list_create(max, L);
    nodo_min = list_create(min, nodo_max);
    return nodo_min;    
}

/**
 ** Le funzioni seguenti sono gia' implementate 
 **/

/* Crea (mediante malloc() ) e restituisce un puntatore ad un nuovo
   nodo di una lista; il nodo contiene il valore v e punta a t come
   elemento successivo. Il chiamante e' responsabile per deallocare
   mediante free() o simili la zona di memoria restituita da questa
   funzione, quando non piu' utilizzata */
struct list  *list_create(int v, struct list *t)
{
    struct list *r=(struct list *)malloc(sizeof(struct list));
    r->val = v;
    r->next = t;
    return r;
}

/* Restituisce la lunghezza (numero di nodi) della lista L; se L e' la
   lista vuota, restituisce 0 */
int list_length(struct list *L)
{
    /* Versione ricorsiva */
    if ( NULL == L ) {
        return 0;
    } else {
        return (1 + list_length(L->next));
    }
}

/* Libera la memoria occupata da tutti i nodi della lista L */
void list_destroy(struct list *L)
{
    /* versione ricorsiva */
    if ( L != NULL ) {
        list_destroy(L->next);
        L->next = NULL; /* non necessario... */
        free(L);
    }
}

/* Restituisce 1 se e solo se L e' la lista vuota, 0 altrimenti */
int is_empty(struct list *L)
{
    return (NULL == L);
}

/* Stampa il contenuto della lista L */
void list_print(struct list *L)
{
    printf("(");
    while (L != NULL) {
        printf("%d", L->val);
        if (L->next != NULL) printf(", ");
        L = L->next;
    }
    printf(")");
}

int list_equal(struct list *L1, struct list *L2)
{
    if (is_empty(L1) || is_empty(L2)) {
        /* Se siamo qui significa che almeno una tra L1 e L2 e' vuota
           (possono esserlo entrambe). Allora le liste sono uguali se
           e solo se sono ENTRAMBE vuote */
        return (is_empty(L1) && is_empty(L2));
    } else {
        if (L1->val != L2->val) {
            return 0;
        } else {
            return list_equal(L1->next, L2->next);
        }
    }
}

struct list *list_from_array(int v[], int n)
{
    if ( n == 0 ) {
        return NULL;
    } else {
        return list_create( v[0], list_from_array(v+1, n-1));
    }
}

void test(struct list *L, struct list *expect)
{
    struct list *result = f(L);

    if (list_equal(result, expect)) {
        printf("Test OK\n");
    } else {
        printf("Test FALLITO: risultato atteso=");
        list_print(expect);
        printf(", risultato ottenuto=");
        list_print(result);
        printf("\n");
    }
}

#define LIST_FROM_ARRAY(a) list_from_array(a, (sizeof(a)/sizeof(a[0])))

int main( void )
{
    int in1[] = {13, 2, -7, 21, -7};
    int out1[] = {-7, 21, 13, 2, -7, 21, -7};
    /* A lezione non abbiamo visto l'uso della macro in linguaggio C;
       e' sufficiente sapere che l'espressione

       LIST_FROM_ARRAY(A1)

       equivale a scrivere

       list_from_array(A1, (sizeof(A1)/sizeof(A1[0])))
    */
    struct list *L1_in = LIST_FROM_ARRAY(in1);
    struct list *L1_out = LIST_FROM_ARRAY(out1);
    
    int in2[] = {0};
    int out2[] = {0, 0, 0};
    struct list *L2_in = LIST_FROM_ARRAY(in2);
    struct list *L2_out = LIST_FROM_ARRAY(out2);

    int in3[] = {999, 1, 2, -999, 3, 4};
    int out3[] = {-999, 999, 999, 1, 2, -999, 3, 4};
    struct list *L3_in = LIST_FROM_ARRAY(in3);
    struct list *L3_out = LIST_FROM_ARRAY(out3);

    int in4[] = {1, 2, 3, 4, 3, 2, 1};
    int out4[] = {1, 4, 1, 2, 3, 4, 3, 2, 1};
    struct list *L4_in = LIST_FROM_ARRAY(in4);
    struct list *L4_out = LIST_FROM_ARRAY(out4);


    printf("=== Inizio test ===\n");
    test(L1_in, L1_out);
    test(L2_in, L2_out);
    test(L3_in, L3_out);
    test(L4_in, L4_out);
    printf("=== Fine test ===\n");
    
    return 0;
}
